# Generated by Django 2.2.28 on 2023-06-21 12:51

from django.db import migrations

from django.core.management import call_command


def recollect(apps, schema_editor):
    call_command('notifications_collect_stats', verbosity=0)


class Migration(migrations.Migration):

    dependencies = [
        ('wcd_notifications', '0004_auto_20230621_1414'),
    ]

    operations = [
        migrations.RunPython(recollect, migrations.RunPython.noop)
    ]
