# Generated by Django 3.2 on 2023-06-16 11:53

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('wcd_notifications', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='notification',
            options={'ordering': ('-ordering_field', '-sent_at', '-created_at', '-pk'), 'verbose_name': 'Notification', 'verbose_name_plural': 'Notifications'},
        ),
        migrations.RenameField(
            model_name='notification',
            old_name='sended_at',
            new_name='sent_at',
        ),
        migrations.AlterField(
            model_name='notification',
            name='flags',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), blank=True, default=list, size=None, verbose_name='Flags'),
        ),
    ]
